/*
 * Copyright (c) 2001, Adam Dunkels.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Adam Dunkels.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.  
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  
 *
 * This file is part of the uIP TCP/IP stack.
 *
 * $Id: cgi.c,v 1.2 2002/02/05 08:30:34 adam Exp $
 *
 */

/*
 * This file includes functions that are called by the web server
 * scripts. The functions take no argument, and the return value is
 * interpreted as follows. A zero means that the function did not
 * complete and should be invoked for the next packet as well. A
 * non-zero value indicates that the function has completed and that
 * the web server should move along to the next script line.
 *
 */

#include "uip.h"
#include "cgi.h"
#include "httpd.h"
#include "fs.h"

#include <dsensor.h>
#include <dmotor.h>
#include <dlcd.h>

static u8_t sensor(void);
static u8_t motor(void);

cgifunction cgitab[] = {
  sensor,
  motor
};

/*-----------------------------------------------------------------------------------*/
static u8_t
sensor(void)
{
  u8_t s;

  if(uip_flags & UIP_ACKDATA) {
    return 1;
  }
  
  switch(hs->script[4]) {
  case '1':
    s = TOUCH_1;
    break;
  case '2':
    s = TOUCH_2;
    break;
  case '3':
    s = TOUCH_3;
    break;
  default:
    s = 0;
    break;
  }
  if(s) {
    uip_appdata = "on  ";
  } else {
    uip_appdata = "off ";
  }
  uip_len = 4;
  return 0;
}
/*-----------------------------------------------------------------------------------*/
static u8_t
motor(void)
{
  u8_t dir;

  cputs("motor");
  
  if(hs->script[5] == 'f') {
    dir = fwd;
  } else if(hs->script[5] == 'b') {
    dir = rev;
  } else {
    dir = off;
  }      
  
  if(hs->script[4] == 'a') {
    motor_a_dir(dir);
    motor_a_speed(255);
  } else if(hs->script[4] == 'b') {
    motor_b_dir(dir);
    motor_b_speed(255);
  } else if(hs->script[4] == 'c') {
    motor_c_dir(dir);
    motor_c_speed(255);
  }
  
  return 1;
}
/*-----------------------------------------------------------------------------------*/
